"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const color_1 = require("@heroku-cli/color");
const core_1 = require("@oclif/core");
const cli_ux_1 = require("cli-ux");
const core_2 = require("@salesforce/core");
const flags_1 = require("../../../lib/flags");
const base_1 = require("../../../lib/base");
const utils_1 = require("../../../lib/utils");
core_2.Messages.importMessagesDirectory(__dirname);
const messages = core_2.Messages.loadMessages('@salesforce/plugin-functions', 'env.var.set');
class ConfigSet extends base_1.default {
    parseKeyValuePairs(pairs) {
        if (pairs.length === 0) {
            this.error('Usage: sfdx env:var:set KEY1=VALUE1 [KEY2=VALUE2 ...]\nMust specify KEY and VALUE to set.');
        }
        return pairs.reduce((acc, elem) => {
            if (elem.indexOf('=') === -1) {
                this.error(`${color_1.default.cyan(elem)} is invalid. Please use the format ${color_1.default.cyan('key=value')}`);
            }
            const equalsIndex = elem.indexOf('=');
            const key = elem.slice(0, equalsIndex);
            const value = elem.slice(equalsIndex + 1);
            return Object.assign(Object.assign({}, acc), { [key]: value });
        }, {});
    }
    parseErrors(errorObject, targetCompute) {
        // may have create a cleaner way to parse the errors
        // to fit the sfdx json error object, but this currently works
        const errObj = errorObject.stack;
        const isInvalidEnvironment = errObj.includes("Error: Couldn't");
        const hasNoInput = errObj.includes('Error: Usage:');
        const isInvalidInput = !hasNoInput;
        if (isInvalidEnvironment) {
            const errorStackStartIndex = errObj.indexOf('at');
            const errStack = errObj.substr(errorStackStartIndex);
            errorObject.message = `Couldn't find that app <${targetCompute}>`;
            errorObject.stack = errStack;
            this.error(errObj);
        }
        if (isInvalidInput) {
            const errorIndex = errObj.indexOf(':') + 1;
            const keyValueIndex = errObj.indexOf('value') + 5;
            // eslint-disable-next-line no-control-regex
            const errMessage = errObj
                .substr(errorIndex, keyValueIndex)
                // eslint-disable-next-line no-control-regex
                .replace(/\u001b\[.*?m/g, '')
                .replace('\n', '')
                .replace(' ', '');
            // eslint-disable-next-line no-control-regex
            const errStack = errObj
                .substr(keyValueIndex)
                // eslint-disable-next-line no-control-regex
                .replace(/\u001b\[.*?m\r?\n|\r/g, '')
                .replace('    ', '');
            errorObject.message = errMessage;
            errorObject.stack = errStack;
            this.error(errObj);
        }
        if (hasNoInput) {
            const errorStackStartIndex = errObj.indexOf('at');
            const errStack = errObj.substr(errorStackStartIndex);
            errorObject.message = 'Must specify KEY and VALUE to set.';
            errorObject.stack = errStack;
            this.error(errObj);
        }
    }
    async run() {
        var _a;
        const { flags, argv } = await this.parse(ConfigSet);
        this.postParseHook(flags);
        // We support both versions of the flag here for the sake of backward compat
        const targetCompute = (_a = flags['target-compute']) !== null && _a !== void 0 ? _a : flags.environment;
        if (!targetCompute) {
            throw new core_1.Errors.CLIError(`Missing required flag:
        -e, --target-compute TARGET-COMPUTE  ${color_1.default.dim('Environment name.')}
       See more help with --help`);
        }
        if (flags.environment) {
            cli_ux_1.cli.warn(messages.getMessage('flags.environment.deprecation'));
        }
        const appName = await (0, utils_1.resolveAppNameForEnvironment)(targetCompute);
        if (flags.json) {
            try {
                const configPairs = this.parseKeyValuePairs(argv);
                await this.client.patch(`/apps/${appName}/config-vars`, {
                    data: configPairs,
                });
                cli_ux_1.cli.styledJSON({
                    status: 0,
                    result: null,
                    warnings: [],
                });
                return;
            }
            catch (err) {
                this.parseErrors(err, targetCompute);
            }
        }
        else {
            const configPairs = this.parseKeyValuePairs(argv);
            cli_ux_1.cli.action.start(`Setting ${Object.keys(configPairs)
                .map((key) => color_1.default.configVar(key))
                .join(', ')} and restarting ${color_1.default.app(targetCompute)}`);
            await this.client.patch(`/apps/${appName}/config-vars`, {
                data: configPairs,
            });
            cli_ux_1.cli.action.stop();
        }
    }
}
exports.default = ConfigSet;
ConfigSet.strict = false;
ConfigSet.summary = messages.getMessage('summary');
ConfigSet.examples = messages.getMessages('examples');
ConfigSet.flags = {
    'target-compute': flags_1.FunctionsFlagBuilder.environment({
        exclusive: ['environment'],
    }),
    environment: flags_1.FunctionsFlagBuilder.environment({
        char: 'e',
        exclusive: ['target-compute'],
        hidden: true,
    }),
    json: flags_1.FunctionsFlagBuilder.json,
};
